package org.prlab.idic.privacypolicy;

import android.app.LoaderManager;
import android.content.AsyncTaskLoader;
import android.content.Intent;
import android.content.Loader;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.view.View;
import android.widget.Toast;

import com.google.gson.Gson;
import com.google.gson.JsonSyntaxException;

import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;

import org.prlab.idic.privacypolicy.adapter.InfoItemRecyclerViewAdapter;
import org.prlab.idic.privacypolicy.adapter.PolicyRecyclerViewAdapter;
import org.prlab.idic.privacypolicy.pojo.report.InfoItem;
import org.prlab.idic.privacypolicy.pojo.report.PrivacyPolicy;
import org.prlab.idic.privacypolicy.pojo.report.PrivacyPolicyReport;
import org.prlab.idic.privacypolicy.storage.PrivacyPolicyStorageImpl;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import static org.prlab.idic.privacypolicy.storage.PrivacyPolicyContract.ACCEPT;

/**
 * Created by IDIC on 2017/2/16.
 */
public class ReportActivity extends AppCompatActivity implements
        LoaderManager.LoaderCallbacks<PrivacyPolicyReport>,
        PolicyRecyclerViewAdapter.ItemCheckedChangedListener {

    private static final String TAG = ReportActivity.class.getSimpleName();

    private RecyclerView mDescriptionRecyclerView;
    private RecyclerView mPoliciesRecyclerView;

    private Gson gson = new Gson();

    private PrivacyPolicyStorageImpl mPolicyStorage;
    private PrivacyPolicyReport mPrivacyPolicyReport;
    private String mReportId;

    private static final int PRIVACY_POLICY_LOADER_ID = 87;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_report);
        resolveData();
        initView();
    }

    private void resolveData() {
        Intent intent = getIntent();
        mReportId = intent.getStringExtra(getString(R.string.intent_report_id_key));
        mPolicyStorage = new PrivacyPolicyStorageImpl(getContentResolver(), intent.getStringExtra(getString(R.string.intent_report_mac_key)));
        getLoaderManager().initLoader(PRIVACY_POLICY_LOADER_ID, null, this);
    }

    private void initView() {
        mDescriptionRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_report_description);
        mPoliciesRecyclerView = (RecyclerView) findViewById(R.id.recycler_view_report_policies);
    }

    private void setDescriptionRecyclerView(String version, String description) {
        List<InfoItem> items = new LinkedList<>();
        items.add(new InfoItem()
                .setKey(getString(R.string.policy_report_version_title))
                .setValue(version));
        items.add(new InfoItem()
                .setKey(getString(R.string.policy_report_description_title))
                .setValue(description));
        mDescriptionRecyclerView.setAdapter(new InfoItemRecyclerViewAdapter(items, R.layout.recycler_view_item_horizontally));
    }

    private void setContentRecyclerView(List<PrivacyPolicy> policies,
                                        List<Integer> selections,
                                        PolicyRecyclerViewAdapter.ItemCheckedChangedListener itemCheckedChangedListener) {
        mPoliciesRecyclerView.setAdapter(new PolicyRecyclerViewAdapter(
                getResources(), policies, selections, itemCheckedChangedListener
                /*, android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M ?
                        getColor(R.color.colorAnother) : getResources().getColor(R.color.colorAnother)*/));
    }

    @Override
    public Loader<PrivacyPolicyReport> onCreateLoader(int id, Bundle args) {
        return new AsyncTaskLoader<PrivacyPolicyReport>(this) {

            @Override
            protected void onStartLoading() {
                if (null == mPrivacyPolicyReport) {
                    forceLoad();
                }
            }

            @Override
            public PrivacyPolicyReport loadInBackground() {
                OkHttpClient client = new OkHttpClient();
                Request request = new Request.Builder()
                        .url(getString(R.string.get_policy_url))
                        .post(new FormBody.Builder()
                                .add(getString(R.string.get_policy_id_parameter_key), mReportId)
                                .build())
                        .build();
                try {
                    return gson.fromJson(client.newCall(request).execute().body().string(), PrivacyPolicyReport.class);
                } catch (IOException | JsonSyntaxException e) {
                    e.printStackTrace();
                    return null;
                }
            }

        };
    }

    @Override
    public void onLoadFinished(Loader<PrivacyPolicyReport> loader, PrivacyPolicyReport report) {
        if (null != report) {
            mPrivacyPolicyReport = report;
            setDescriptionRecyclerView(report.getVersion(), report.getDescription());
            setContentRecyclerView(report.getPolicies(), getSelections(report.getPolicies()), this);
        } else {
            Toast.makeText(this, R.string.message_report_not_found, Toast.LENGTH_LONG).show();
            finish();
        }
    }

    private List<Integer> getSelections(List<PrivacyPolicy> policies) {
        List<Integer> selections = new LinkedList<>();
        for (PrivacyPolicy policy : policies) {
            if (ACCEPT.equals(mPolicyStorage.get(policy.getId()))) {
                selections.add(policy.getId());
            }
        }
        return selections;
    }

    @Override
    public void onLoaderReset(Loader<PrivacyPolicyReport> loader) {

    }

    public void onDeviceInfoButtonClicked(View view) {
        Intent intent = new Intent();
        intent.setClass(ReportActivity.this, DialogDeviceInfoActivity.class);
        intent.putExtra(getString(R.string.intent_device_info_key), mPrivacyPolicyReport.getDevice());
        startActivity(intent);
    }

    @Override
    public void onItemCheckedChanged(PrivacyPolicy policy, boolean isChecked) {
        if (isChecked) {
            mPolicyStorage.accept(policy.getId());
        } else {
            mPolicyStorage.decline(policy.getId());
        }
    }

}
