package org.prlab.idic.privacypolicy.adapter;

import android.content.res.Resources;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.ImageView;
import android.widget.Switch;
import android.widget.TextView;

import org.prlab.idic.privacypolicy.R;
import org.prlab.idic.privacypolicy.pojo.report.InfoItem;
import org.prlab.idic.privacypolicy.pojo.report.PrivacyPolicy;
import org.prlab.idic.privacypolicy.pojo.report.p3p.Datum;
import org.prlab.idic.privacypolicy.pojo.report.p3p.Purpose;
import org.prlab.idic.privacypolicy.pojo.report.p3p.Recipient;
import org.prlab.idic.privacypolicy.pojo.report.p3p.Remedy;
import org.prlab.idic.privacypolicy.pojo.report.p3p.Statement;

import java.util.LinkedList;
import java.util.List;

public class PolicyRecyclerViewAdapter
        extends RecyclerView.Adapter<PolicyRecyclerViewAdapter.ViewHolder> {

    private static final String POLICY_PREFIX = "Policy ";

    private List<PrivacyPolicy> policies;
    private List<Integer> selections;
    private ItemCheckedChangedListener itemCheckedChangedListener;
    private int separatedColor;

    private final String SEPARATOR;

    //BASE
    private final String COLLECTOR_NAME;
    private final String COLLECTOR_PHONE;
    private final String COLLECTOR_EMAIL;
    private final String DISPUTE_RELATED_ORGANIZATION;
    private final String DISPUTE_RESOLUTION_TYPE;
    private final String ACCESS;

    //Statement
    private final String CONSEQUENCE;
    private final String PURPOSE_TYPE;
    private final String PURPOSE_DESCRIPTION;
    private final String DATUM_TYPE;
    private final String DATUM_DESCRIPTION;
    private final String RECIPIENT_TYPE;
    private final String RECIPIENT_DESCRIPTION;
    private final String REMEDIES;
    private final String RETENTION;

    public PolicyRecyclerViewAdapter(Resources resources,
                                     List<PrivacyPolicy> policies,
                                     List<Integer> selections,
                                     ItemCheckedChangedListener itemCheckedChangedListener){
        this(resources, policies, selections, itemCheckedChangedListener, -1);
    }

    public PolicyRecyclerViewAdapter(Resources resources,
                                     List<PrivacyPolicy> policies,
                                     List<Integer> selections,
                                     ItemCheckedChangedListener itemCheckedChangedListener,
                                     int separatedColor) {
        SEPARATOR = resources.getString(R.string.policy_separator);

        //BASE
        String COLLECTOR = resources.getString(R.string.policy_collector_title);
        COLLECTOR_NAME = COLLECTOR + SEPARATOR + resources.getString(R.string.policy_collector_name);
        COLLECTOR_PHONE = COLLECTOR + SEPARATOR + resources.getString(R.string.policy_collector_phone);
        COLLECTOR_EMAIL = COLLECTOR + SEPARATOR + resources.getString(R.string.policy_collector_email);
        String DISPUTES = resources.getString(R.string.policy_dispute_title);
        DISPUTE_RESOLUTION_TYPE = DISPUTES + SEPARATOR + resources.getString(R.string.policy_dispute_resolution_type);
        DISPUTE_RELATED_ORGANIZATION = DISPUTES + SEPARATOR + resources.getString(R.string.policy_dispute_related_organization);
        ACCESS = resources.getString(R.string.policy_access_title);

        //Statement
        CONSEQUENCE = resources.getString(R.string.policy_statement_consequence_title);
        String PURPOSES = resources.getString(R.string.policy_statement_purpose_title);
        PURPOSE_TYPE = PURPOSES + SEPARATOR + resources.getString(R.string.policy_statement_purpose_type);
        PURPOSE_DESCRIPTION = PURPOSES + SEPARATOR + resources.getString(R.string.policy_statement_purpose_description);
        String DATA = resources.getString(R.string.policy_statement_data_title);
        DATUM_TYPE = DATA + SEPARATOR + resources.getString(R.string.policy_statement_data_type);
        DATUM_DESCRIPTION = DATA + SEPARATOR + resources.getString(R.string.policy_statement_data_description);
        String RECIPIENTS = resources.getString(R.string.policy_statement_recipient_title);
        RECIPIENT_TYPE = RECIPIENTS + SEPARATOR + resources.getString(R.string.policy_statement_recipient_type);
        RECIPIENT_DESCRIPTION = RECIPIENTS + SEPARATOR + resources.getString(R.string.policy_statement_recipient_description);
        REMEDIES = resources.getString(R.string.policy_statement_remedies);
        RETENTION = resources.getString(R.string.policy_statement_retention);

        this.policies = policies;
        this.selections = selections;
        this.itemCheckedChangedListener = itemCheckedChangedListener;
        this.separatedColor = separatedColor;
    }

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(parent.getContext())
                .inflate(R.layout.recycler_view_policy, parent, false));
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        PrivacyPolicy policy = policies.get(position);
        String title = POLICY_PREFIX + policy.getId();
        holder.mPolicyTitleTextView.setText(title);
        holder.mPolicyDescriptionTextView.setText(policy.getDescription());
        holder.mPolicyCheckBox.setChecked(selections.contains(policy.getId()));
        List<InfoItem> items = new LinkedList<>();
        addItem(items, COLLECTOR_NAME, policy.getCollector().getName());
        addItem(items, COLLECTOR_EMAIL, policy.getCollector().getEmail());
        addItem(items, COLLECTOR_PHONE, policy.getCollector().getPhone());
        addItem(items, DISPUTE_RESOLUTION_TYPE, policy.getDispute().getType().name());
        addItem(items, DISPUTE_RELATED_ORGANIZATION, policy.getDispute().getRelatedOrganization());
        addItem(items, ACCESS, policy.getAccess().name());

        StringBuilder remedies = new StringBuilder();
        for (Remedy remedy : policy.getRemedies()) {
            remedies.append(remedy.getType()).append(SEPARATOR);
        }
        if (remedies.length() > 0)
            addItem(items, REMEDIES, remedies.toString());

        for (Statement statement : policy.getStatements()) {
//            addItem(items, CONSEQUENCE, statement.getConsequence());
            for (Purpose purpose : statement.getPurposes()) {
                addItem(items, PURPOSE_TYPE, purpose.getType().name());
                addItem(items, PURPOSE_DESCRIPTION, purpose.getDescription());
            }
            for (Datum datum : statement.getData()) {
                addItem(items, DATUM_TYPE, datum.getType().name());
                addItem(items, DATUM_DESCRIPTION, datum.getDescription());
            }
            for (Recipient recipient : statement.getRecipients()) {
                addItem(items, RECIPIENT_TYPE, recipient.getType().name());
                addItem(items, RECIPIENT_DESCRIPTION, recipient.getDescription());
            }
            addItem(items, RETENTION, statement.getRetention().name());

        }
        holder.mPolicyContentRecyclerView.setAdapter(new InfoItemRecyclerViewAdapter(items, R.layout.recycler_view_policy_content, separatedColor));
    }

    private void addItem(List<InfoItem> items, String key, String value) {
        items.add(new InfoItem()
                .setKey(key)
                .setValue(value));
    }

    @Override
    public int getItemCount() {
        return policies != null ? policies.size() : 0;
    }

    class ViewHolder extends RecyclerView.ViewHolder {

        private TextView mPolicyTitleTextView;
        private TextView mPolicyDescriptionTextView;
        private Switch mPolicyCheckBox;
        private ImageView mPolicyScaledImageView;
        private RecyclerView mPolicyContentRecyclerView;

        ViewHolder(View itemView) {
            super(itemView);
            mPolicyTitleTextView = (TextView) itemView.findViewById(R.id.text_policy_title);
            mPolicyDescriptionTextView = (TextView) itemView.findViewById(R.id.text_policy_description);
            mPolicyCheckBox = (Switch) itemView.findViewById(R.id.switch_policy_accept);
            mPolicyScaledImageView = (ImageView) itemView.findViewById(R.id.image_view_policy_scaled);
            mPolicyContentRecyclerView = (RecyclerView) itemView.findViewById(R.id.recycler_view_policy_content);
            itemView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    if (mPolicyContentRecyclerView.getVisibility() == View.VISIBLE) {
                        mPolicyContentRecyclerView.setVisibility(View.GONE);
                        mPolicyScaledImageView.setImageResource(R.drawable.ic_arrow_drop_up_black_24dp);
                    } else {
                        mPolicyContentRecyclerView.setVisibility(View.VISIBLE);
                        mPolicyScaledImageView.setImageResource(R.drawable.ic_arrow_drop_down_black_24dp);
                    }
                }
            });
            mPolicyCheckBox.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                    itemCheckedChangedListener.onItemCheckedChanged(policies.get(getAdapterPosition()), isChecked);
                }
            });
        }

    }

    public interface ItemCheckedChangedListener {
        void onItemCheckedChanged(PrivacyPolicy policy, boolean isChecked);
    }

}
