package org.prlab.idic.privacypolicy.storage;

import android.content.ContentResolver;
import android.content.ContentValues;
import android.database.Cursor;
import android.net.Uri;
import android.util.Log;

import org.prlab.idic.privacypolicy.tool.BaseUtils;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import static org.prlab.idic.privacypolicy.storage.PrivacyPolicyContract.ACCEPT;
import static org.prlab.idic.privacypolicy.storage.PrivacyPolicyContract.DECLINE;

/**
 * Created by IDIC on 2017/2/15.
 */
public class PrivacyPolicyStorageImpl implements StorageInterface<Integer, String> {

    private static final String TAG = PrivacyPolicyStorageImpl.class.getSimpleName();

    private ContentResolver mContentResolver;
    private String mTargetDeviceMAC;

    private static final String[] PRIVACY_POLICY_PROJECTION = {
            PrivacyPolicyContract.PolicyEntry.COLUMN_MAC,
            PrivacyPolicyContract.PolicyEntry.COLUMN_POLICY,
            PrivacyPolicyContract.PolicyEntry.COLUMN_PREFERENCE
    };

    public static final int INDEX_POLICY_MAC = 0;
    public static final int INDEX_POLICY_POLICY = 1;
    public static final int INDEX_POLICY_PREFERENCE = 2;

    public PrivacyPolicyStorageImpl(ContentResolver contentResolver, String targetDeviceMAC) {
        this.mContentResolver = contentResolver;
        this.mTargetDeviceMAC = BaseUtils.shortMAC(targetDeviceMAC);
    }

    @Override
    public StorageInterface decline(Integer id) {
        choose(id, DECLINE);
        return this;
    }

    @Override
    public StorageInterface accept(Integer id) {
        choose(id, ACCEPT);
        return this;
    }

    private void choose(Integer id, String choice) {
        ContentValues insertContentValues = new ContentValues();
        insertContentValues.put(PrivacyPolicyContract.PolicyEntry.COLUMN_MAC, mTargetDeviceMAC);
        insertContentValues.put(PrivacyPolicyContract.PolicyEntry.COLUMN_POLICY, id);
        insertContentValues.put(PrivacyPolicyContract.PolicyEntry.COLUMN_PREFERENCE, choice);
        Uri newUri = mContentResolver.insert(
                PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                insertContentValues
        );

        if (newUri == null) {
            ContentValues updateContentValues = new ContentValues();
            updateContentValues.put(PrivacyPolicyContract.PolicyEntry.COLUMN_PREFERENCE, choice);

            int count = mContentResolver.update(
                    PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                    updateContentValues,
                    PrivacyPolicyContract.PolicyEntry.COLUMN_MAC + " = ? " + " AND " +
                            PrivacyPolicyContract.PolicyEntry.COLUMN_POLICY + " = ? ",
                    new String[]{mTargetDeviceMAC, String.valueOf(id)}
            );
            Log.i(TAG, "Update " + String.valueOf(choice) + " to [" + id + "] " + (count == 1 ? "success" : "failure") + ".");
        } else {
            Log.i(TAG, "Put " + String.valueOf(choice) + " to [" + id + "] success.");
        }
    }

    @Override
    public String get(Integer id) {
        String result = null;
        try (Cursor cursor = mContentResolver.query(PrivacyPolicyContract.PolicyEntry.CONTENT_URI,
                PRIVACY_POLICY_PROJECTION,
                PrivacyPolicyContract.PolicyEntry.COLUMN_MAC + " = ? " + " AND " +
                        PrivacyPolicyContract.PolicyEntry.COLUMN_POLICY + " = ? ",
                new String[]{mTargetDeviceMAC, String.valueOf(id)},
                null)) {
            if (cursor != null && cursor.getCount() == 1) {
                cursor.moveToFirst();
                result = cursor.getString(INDEX_POLICY_PREFERENCE);
            }
        }
        return result;
    }

}
